<script type="text/javascript">

// ---- Here is a bit of test code. You can change the value of orig in the first line
// ---- and the value of the escape character in the toZvvyCode call in the second line
// ---- then execute this to see the encoded string and the original string after decoding.

var orig = "He's #bad!";
var code = toZvvyCode(orig, 124);
var decode = fromZvvyCode(code);

document.write("Orig: " + orig + "<br>");
document.write("Code: " + code + "<br>");
document.write("Deco: " + decode.text + "  Error: " + decode.error + "<br>");



//
// ZvvyCode:  Universal safe text encoding method
//
// 		By David R. Woolley
// 		Thinkofit, Inc.
//		www.thinkofit.com
//
//
// ZvvyCode is a text encoding method that converts virtually any text into a format that can
// be handled by virtually any programming language without the danger of there being characters
// embedded in the encoded text that have syntactic meaning in the language. An encoded string
// consists only of the ASCII letters (a-z and A-Z) and digits (0-9), plus one special escape
// character which is designated at the time of encoding and may be anything other than the
// ASCII letters and digits. An encoded string can be decoded without advance knowledge of the
// escape character, because the escape character is read from the beginning of the encoded string.
//
// The original ASCII letters (a-z and A-Z) and digits (0-9) are kept in their original form.
// ALL other characters, including spaces, punctuation, extended ASCII codes, Unicode characters,
// etc. are converted to their hexadecimal character codes bounded on each end by a uniform
// escape character. The escape character may be any character except the ASCII letters and digits,
// so you can choose an escape character that will not cause trouble in your application.
//
// An encoded string begins with an 8-character sequence consisting of two instances of the escape
// character used in encoding the string, the letters "zvvy", and two more instances of the escape
// character. The decoder reads these characters to determine (1) whether the string is, in fact,
// a ZvvyCode string, and (2) what character to interpret as the escape character for the decoding
// process.
//
// If the first eight characters do not follow this pattern exactly, or if the characters in the
// positions designated for the escape characters are ASCII letters or digits, then the string
// is considered to be NOT ZvvyCode, and is returned as-is, without decoding.
//
// Example: Suppose you choose a slash as the escape character to encode this text:
//
// 		I said, "Joe's bag is 3/4 full!"
//
// The ZvvyCode encoding would be:
//
//		//zvvy//I/20/said/2c//20//22/Joe/27/s/20/bag/20/is/20/3/2f/4/20/full/21//22/
//
//	Note that it does not matter that the original text has a slash in it, even though the slash
//	is to be used as the escape character. The slash is simply converted to /2f/.
//
//	The escape character does not have to be a visible character at all. It can be, for example,
//	0 (ASCII null), 127 (ASCII del), 27 (ASCII esc), 255 (the last extended ASCII character)
//	or even a large Unicode character value. The only restriction is that it cannot be one of
//	the ASCII upper or lower case letters or numeric digits.
//


//
//	toZvvyCode: Convert a string to ZvvyCode encoding.
//
//	Arguments:
//		str		String to be encoded
//		escChr	Character code to use as the escape character, given as a numeric value.
//				(E.g., 92 or 0x5c to use backslash as the escape.)
//				MUST NOT BE A LETTER (a-z, A-Z) OR DIGIT (0-9). Any other character code is valid.
//
//	Returns:
//		ZvvyCode encoded string
//
//
function toZvvyCode (str, escChr) {

	var i, chr;
	var lth = str.length;
	var escChrStr = String.fromCharCode(escChr);

	// Begin with two escape characters, the "zvvy" identifier, and two more escapes.
	var rs = escChrStr + escChrStr + "zvvy" + escChrStr + escChrStr;

	for (i = 0; i < lth; i++) {
		chr = str.charCodeAt (i);
		if ((chr >= 0x30 && chr <= 0x39) || (chr >= 0x41 && chr <= 0x5A) || (chr >= 0x61 && chr <= 0x7A)) {
			// Copy a-z, A-Z, and 0-9 as-is.
			rs += str.charAt(i);
		}
		else {
			// Output a hex character value surrounded by escape characters.
			rs += escChrStr + chr.toString(16) + escChrStr;
		}
	}
	return (rs);
}


//
//	fromZvvyCode: Convert a ZvvyCode encoded string back to its original value.
//
//	Arguments:
//		str		String to be decoded
//
//	Returns:
//		Object with two properties:
//			text:	Decoded string
//			error:	-1 if successful
//					 0 if the string was determined to be not encoded and is returned unchanged
//					 2 or greater if the encoded string is malformed
//
//	An encoded string is considered malformed if any of the following occur:
//		* A value enclosed between two escape characters is not a valid hexadecimal number
//		* An opening escape character is not matched by a closing escape - i.e., the string ends
//		  before a terminating escape character is found.
//
//	In case of a malformed string, the returned object will contain:
//			text:	As much of the string as could be decoded before the error was encountered
//			error:	Position within the string where a problematic escape sequence began
//

function fromZvvyCode (str) {

	var i, escPos, escVal, escChr, escChrStr;
	var lth = str.length;
	var retobj = { "text": '', "error": -1 };

	if (lth < 8) {
		// If string is less than eight characters, it cannot be ZvvyCode, so return as is.
		retobj.text = str;
		retobj.error = 0;
		return (retobj);
	}

	escChr = str.charCodeAt (0);	// First character should be the escape character used to encode this string
	escChrStr = str.charAt(0);
	if ((escChr >= 0x30 && escChr <= 0x39) || (escChr >= 0x41 && escChr <= 0x5A) || (escChr >= 0x61 && escChr <= 0x7A)) {
		// First character is a letter or digit, not a valid escape character -- means this is not ZvvyCode so return as is.
		retobj.text = str;
		retobj.error = 0;
		return (retobj);
	}
	if (escChrStr != str.charAt(1) || escChrStr != str.charAt(6) || escChrStr != str.charAt(7) || str.substring(2,6) != "zvvy") {
		// First 8 characters do not follow ZvvyCode pattern -- means string is not ZvvyCode, so return as is.
		retobj.text = str;
		retobj.error = 0;
		return (retobj);
	}

	i = 8;		// Start decoding after the first eight characters
	while (i < lth) {
		escPos = str.indexOf(escChrStr, i);
		if (escPos < 0) {
			// No more instances of the escape string found, so append remainder of string and we are done.
			retobj.text += str.substring(i);
			return ( retobj );
		}
		retobj.text += str.substring(i, escPos);	// Append all regular characters up to where escape was found
		i = escPos + 1;						// Point to first character beyond the escape
		escPos = str.indexOf(escChrStr, i);	// Find terminating escape
		if (escPos >= 0) {
			escVal = '0x' + str.substring (i, escPos);
			if (isNaN(escVal)) {
				// Value in the escape sequence is not a valid number
				retobj.error = i - 1;
				return (retobj);
			}
			retobj.text += String.fromCharCode(escVal);
			i = escPos + 1;
		}
		else {

			// Escape sequence was started but string ended with no terminating escape found.
			// This indicates a malformed string.
			// Return decoded text up to the beginning of the unterminated escape sequence,
			// and set error value to the character position where the escape sequence began.

			retobj.error = i - 1;
			return ( retobj );
		}
	}
	return (retobj);
}

</script>